//
// connection.hpp
// ~~~~~~~~~~~~~~
//
//

#ifndef MPRE_CONNECTION_HPP
#define MPRE_CONNECTION_HPP

#include <boost/asio.hpp>
#include <boost/array.hpp>
#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "request_handler.hpp"

namespace mpre {
  namespace network {

    class connection_manager;

    /// Represents a single connection from a client.
    class connection
      : public boost::enable_shared_from_this<connection>,
	private boost::noncopyable
    {
    public:
      /// Construct a connection with the given io_service.
      explicit connection(boost::asio::io_service& io_service,
			  connection_manager& manager, request_handler& handler);

      /// Get the socket associated with the connection.
      boost::asio::ip::tcp::socket& socket();
      
      /// Start the first asynchronous operation for the connection.
      void start();
      
      /// Stop all asynchronous operations associated with the connection.
      void stop();

      // check keepalive
      void check_keepalive();
      

    private:
      /// Handle completion of a read operation.
      void handle_read(const boost::system::error_code& e,
		       std::size_t bytes_transferred);

      /// Handle completion of a write operation.
      void handle_write(const boost::system::error_code& e);
      
    
      // Handle empty read op. If it fails, then there is a problem on socket ( => peer dead! )
      void handle_read_test(const boost::system::error_code& e,
			    std::size_t bytes_transferred);

      /// Socket for the connection.
      boost::asio::ip::tcp::socket socket_;

      /// The manager for this connection.
      connection_manager& connection_manager_;

      /// The handler used to process the incoming request.
      request_handler& request_handler_;

      /// Buffer for incoming data.
      enum { max_length = 8192 };
      boost::array<char, max_length> buffer_;

    };

    typedef boost::shared_ptr<connection> connection_ptr;

  } // namespace network
} // namespace mpre


#endif // MPRE_CONNECTION_HPP
